#!/usr/bin/python3
import struct
import array
import time
from . import i2c_base

HTU21D_ADDR = 0x40
CMD_READ_TEMP_HOLD = b"\xE3"
CMD_READ_HUM_HOLD = b"\xE5"
CMD_READ_TEMP_NOHOLD = b"\xF3"
CMD_READ_HUM_NOHOLD = b"\xF5"
CMD_WRITE_USER_REG = b"\xE6"
CMD_READ_USER_REG = b"\xE7"
CMD_SOFT_RESET = b"\xFE"


class HTU21D(object):
    def __init__(self):
        self.dev = i2c_base.i2c(HTU21D_ADDR, 1)  # HTU21D 0x40, bus 1
        self.dev.write(CMD_SOFT_RESET)  # Soft reset
        time.sleep(.1)

    def ctemp(self, sensor_temp):
        t_sensor_temp = sensor_temp / 65536.0
        return -46.85 + (175.72 * t_sensor_temp)

    def chumid(self, sensor_humid):
        t_sensor_humid = sensor_humid / 65536.0
        return -6.0 + (125.0 * t_sensor_humid)

    def temp_coefficient(self, rh_actual, temp_actual, coefficient=-0.15):
        return rh_actual + (25 - temp_actual) * coefficient

    def crc8check(self, value):
        # Ported from Sparkfun Arduino HTU21D Library:
        # https://github.com/sparkfun/HTU21D_Breakout
        remainder = ((value[0] << 8) + value[1]) << 8
        remainder |= value[2]

        # POLYNOMIAL = 0x0131 = x^8 + x^5 + x^4 + 1 divisor =
        # 0x988000 is the 0x0131 polynomial shifted to farthest
        # left of three bytes
        divisor = 0x988000

        for i in range(0, 16):
            if(remainder & 1 << (23 - i)):
                remainder ^= divisor
            divisor = divisor >> 1

        if remainder == 0:
            return True
        else:
            return False

    def read_temperature(self):
        self.dev.write(CMD_READ_TEMP_NOHOLD)  # Measure temp
        time.sleep(.1)
        data = self.dev.read(3)
        buf = array.array('B', data)
        if self.crc8check(buf):
            temp = (buf[0] << 8 | buf[1]) & 0xFFFC
            return self.ctemp(temp)
        else:
            return -255

    def read_humidity(self):
        temp_actual = self.read_temperature()  # For temperature coefficient compensation
        self.dev.write(CMD_READ_HUM_NOHOLD)  # Measure humidity
        time.sleep(.1)
        data = self.dev.read(3)
        buf = array.array('B', data)

        if self.crc8check(buf):
            humid = (buf[0] << 8 | buf[1]) & 0xFFFC
            rh_actual = self.chumid(humid)

            rh_final = self.temp_coefficient(rh_actual, temp_actual)

            rh_final = 100.0 if rh_final > 100 else rh_final  # Clamp > 100
            rh_final = 0.0 if rh_final < 0 else rh_final  # Clamp < 0

            return rh_final
        else:
            return -255

if __name__ == "__main__":
    obj = HTU21D()
    print("Temp: %s C" % round(obj.read_temperature(),1))
    print("Humid: %s %% rH" % round(obj.read_humidity(),1))




from __future__ import print_function

from xaal.lib import Device,Engine,tools,bindings
import logging
import platform
from . import HTU21D
import logging

PACKAGE_NAME = "xaal.htu21d"

logger = logging.getLogger(PACKAGE_NAME)

def build_dev(addr,dev_type):
    dev            = Device(dev_type)
    dev.address    = addr
    dev.vendor_id  = "IHSEV"
    dev.version    = 0.1
    dev.info       = "%s@%s" % (PACKAGE_NAME,platform.node())
    dev.url        = "https://learn.adafruit.com/adafruit-htu21d-f-temperature-humidity-sensor/overview"
    dev.product_id = "HTU21D/I2C"
    return dev
        

def smooth(attribute,value,gap):
    old = attribute.value
    if not old:
        attribute.value = value
        return
    
    if not (old - gap <= value <= old + gap):
        attribute.value = value


class GW:
    def __init__(self,engine):
        self.engine = engine
        self.config()
        self.setup()
        
    def config(self):
        cfg = tools.load_cfg(PACKAGE_NAME)
        if not cfg:
            logger.info('Missing config file, building a new one')
            cfg = tools.new_cfg(PACKAGE_NAME)
            cfg['config']['refresh'] = 1
            cfg['config']['thermometer'] = str(tools.get_random_uuid())
            cfg['config']['hygrometer'] = str(tools.get_random_uuid())
            cfg.write()
        self.cfg = cfg
        
    def setup(self):
        cfg = self.cfg['config']
        rate = float(cfg['refresh'])
        self.engine.add_timer(self.process,rate)

        thermometer_addr = tools.get_uuid(cfg['thermometer'])
        hygrometer_addr = tools.get_uuid(cfg['hygrometer'])

        # Just kidding w/ UUID 
        grp_uuid = bindings.UUID(bytes=thermometer_addr.bytes[:-2] + bytes(2)) + 0xeeff

        dev = build_dev(thermometer_addr,"thermometer.basic")
        dev.group_id = grp_uuid     
        self.temp = dev.new_attribute("temperature")
        self.engine.add_device(dev)
        
        dev = build_dev(hygrometer_addr,"hygrometer.basic")
        dev.group_id = grp_uuid
        self.hum = dev.new_attribute("humidity")
        self.engine.add_device(dev)

        self.sensor = HTU21D.HTU21D()
        
    def process(self):
        smooth(self.temp,round(self.sensor.read_temperature(),1),0.1)
        smooth(self.hum,round(self.sensor.read_humidity(),0),1)
    
def setup(eng):
    gw = GW(eng)
    return True



#!/usr/bin/python3
import io, fcntl 
I2C_SLAVE=0x0703 

class i2c(object):
   def __init__(self, device, bus):
      self.fr = io.open("/dev/i2c-"+str(bus), "rb", buffering=0)
      self.fw = io.open("/dev/i2c-"+str(bus), "wb", buffering=0)
      # set device address
      fcntl.ioctl(self.fr, I2C_SLAVE, device)
      fcntl.ioctl(self.fw, I2C_SLAVE, device)
   def write(self, bytes):
      self.fw.write(bytes)
   def read(self, bytes):
      return self.fr.read(bytes)
   def close(self):
      self.fw.close()
      self.fr.close()
